@@ -113,7 +113,7 @@ class RackRequestTest < Minitest::Spec
   it "wrap the rack variables" do
     req = make_request(Rack::MockRequest.env_for("http://example.com:8080/"))
 
-    req.body.must_respond_to :gets
+    req.body.must_be_nil
     req.scheme.must_equal "http"
     req.request_method.must_equal "GET"
 
@@ -131,7 +131,7 @@ class RackRequestTest < Minitest::Spec
     req.host.must_equal "example.com"
     req.port.must_equal 8080
 
-    req.content_length.must_equal "0"
+    req.content_length.must_be_nil
     req.content_type.must_be_nil
   end
 
@@ -186,6 +186,11 @@ class RackRequestTest < Minitest::Spec
     req.host.must_be_nil
     req.hostname.must_be_nil
 
+    req = make_request \
+      Rack::MockRequest.env_for("/", "HTTP_HOST" => "some_service:3001")
+    req.host.must_equal "some_service"
+    req.hostname.must_equal "some_service"
+
     req = make_request \
       Rack::MockRequest.env_for("/", "SERVER_NAME" => "example.org", "SERVER_PORT" => "9292")
     req.host.must_equal "example.org"
@@ -303,6 +308,10 @@ class RackRequestTest < Minitest::Spec
       Rack::MockRequest.env_for("/", "HTTP_HOST" => "www2.example.org:81")
     req.port.must_equal 81
 
+    req = make_request \
+      Rack::MockRequest.env_for("/", "HTTP_HOST" => "some_service:3001")
+    req.port.must_equal 3001
+
     req = make_request \
       Rack::MockRequest.env_for("/", "SERVER_NAME" => "example.org", "SERVER_PORT" => "9292")
     req.port.must_equal 9292
@@ -579,6 +588,22 @@ class RackRequestTest < Minitest::Spec
     request.params.must_equal "foo" => "bar", "quux" => "bla", "nothing" => nil, "empty" => ""
   end
 
+  it "handles invalid unicode in query string value" do
+    request = make_request(Rack::MockRequest.env_for(qs = "/?foo=%81E"))
+    request.query_string.must_equal "foo=%81E"
+    request.GET.must_equal "foo" => "\x81E"
+    request.POST.must_be :empty?
+    request.params.must_equal "foo" => "\x81E"
+  end
+
+  it "handles invalid unicode in query string key" do
+    request = make_request(Rack::MockRequest.env_for("/?foo%81E=1"))
+    request.query_string.must_equal "foo%81E=1"
+    request.GET.must_equal "foo\x81E" => "1"
+    request.POST.must_be :empty?
+    request.params.must_equal "foo\x81E" => "1"
+  end
+
   it "not truncate query strings containing semi-colons #543 only in POST" do
     mr = Rack::MockRequest.env_for("/",
       "REQUEST_METHOD" => 'POST',
@@ -593,8 +618,7 @@ class RackRequestTest < Minitest::Spec
   it "should use the query_parser for query parsing" do
     c = Class.new(Rack::QueryParser::Params) do
       def initialize(*)
-        super
-        @params = Hash.new{|h, k| h[k.to_s] if k.is_a?(Symbol)}
+        super(){|h, k| h[k.to_s] if k.is_a?(Symbol)}
       end
     end
     parser = Rack::QueryParser.new(c, 100)
@@ -660,8 +684,7 @@ class RackRequestTest < Minitest::Spec
   it "use the query_parser's params_class for multipart params" do
     c = Class.new(Rack::QueryParser::Params) do
       def initialize(*)
-        super
-        @params = Hash.new{|h, k| h[k.to_s] if k.is_a?(Symbol)}
+        super(){|h, k| h[k.to_s] if k.is_a?(Symbol)}
       end
     end
     parser = Rack::QueryParser.new(c, 100)
@@ -696,6 +719,11 @@ class RackRequestTest < Minitest::Spec
       message.must_equal "invalid %-encoding (a%)"
   end
 
+  it "return empty POST data if rack.input is missing" do
+    req = make_request({})
+    req.POST.must_be_empty
+  end
+
   it "parse POST data when method is POST and no content-type given" do
     req = make_request \
       Rack::MockRequest.env_for("/?foo=quux",
@@ -768,81 +796,21 @@ class RackRequestTest < Minitest::Spec
     req.POST.must_equal "foo" => "bar", "quux" => "bla"
   end
 
-  it "get value by key from params with #[]" do
-    req = make_request \
-      Rack::MockRequest.env_for("?foo=quux")
+  it "return values for the keys in the order given from values_at" do
+    req = make_request Rack::MockRequest.env_for("?foo=baz&wun=der&bar=ful")
+
     assert_output(nil, /deprecated/) do
-      req['foo'].must_equal 'quux'
-      req[:foo].must_equal 'quux'
+      req.values_at('foo').must_equal ['baz']
+      req.values_at('foo', 'wun').must_equal ['baz', 'der']
+      req.values_at('bar', 'foo', 'wun').must_equal ['ful', 'baz', 'der']
     end
 
     next if self.class == TestProxyRequest
-    verbose = $VERBOSE
-    warn_arg = nil
-    req.define_singleton_method(:warn) do |*args|
-      warn_arg = args
-    end
-    begin
-      $VERBOSE = true
-      req['foo'].must_equal 'quux'
-      warn_arg.must_equal ["Request#[] is deprecated and will be removed in a future version of Rack. Please use request.params[] instead", { uplevel: 1 }]
-    ensure
-      $VERBOSE = verbose
-    end
-  end
 
-  it "set value to key on params with #[]=" do
-    req = make_request \
-      Rack::MockRequest.env_for("?foo=duh")
-    assert_output(nil, /deprecated/) do
-      req['foo'].must_equal 'duh'
-      req[:foo].must_equal 'duh'
+    capture_warnings(req) do |warnings|
+      req.values_at('foo').must_equal ['baz']
+      warnings.pop.must_equal ["Request#values_at is deprecated and will be removed in a future version of Rack. Please use request.params.values_at instead", { uplevel: 1 }]
     end
-    req.params.must_equal 'foo' => 'duh'
-
-    if req.delegate?
-      skip "delegate requests don't cache params, so mutations have no impact"
-    end
-
-    assert_output(nil, /deprecated/) do
-      req['foo'] = 'bar'
-    end
-    req.params.must_equal 'foo' => 'bar'
-    assert_output(nil, /deprecated/) do
-      req['foo'].must_equal 'bar'
-      req[:foo].must_equal 'bar'
-    end
-
-    assert_output(nil, /deprecated/) do
-      req[:foo] = 'jaz'
-    end
-    req.params.must_equal 'foo' => 'jaz'
-    assert_output(nil, /deprecated/) do
-      req['foo'].must_equal 'jaz'
-      req[:foo].must_equal 'jaz'
-    end
-
-    verbose = $VERBOSE
-    warn_arg = nil
-    req.define_singleton_method(:warn) do |*args|
-      warn_arg = args
-    end
-    begin
-      $VERBOSE = true
-      req['foo'] = 'quux'
-      warn_arg.must_equal ["Request#[]= is deprecated and will be removed in a future version of Rack. Please use request.params[]= instead", { uplevel: 1 }]
-      req.params['foo'].must_equal 'quux'
-    ensure
-      $VERBOSE = verbose
-    end
-  end
-
-  it "return values for the keys in the order given from values_at" do
-    req = make_request \
-      Rack::MockRequest.env_for("?foo=baz&wun=der&bar=ful")
-    req.values_at('foo').must_equal ['baz']
-    req.values_at('foo', 'wun').must_equal ['baz', 'der']
-    req.values_at('bar', 'foo', 'wun').must_equal ['ful', 'baz', 'der']
   end
 
   it "extract referrer correctly" do
@@ -883,7 +851,7 @@ class RackRequestTest < Minitest::Spec
     req.media_type.must_be_nil
   end
 
-  it "cache, but invalidates the cache" do
+  deprecated "cache, but invalidates the cache" do
     req = make_request \
       Rack::MockRequest.env_for("/?foo=quux",
         "CONTENT_TYPE" => "application/x-www-form-urlencoded",
@@ -1331,6 +1299,26 @@ EOF
     f[:filename].must_equal "dj.jpg"
     f.must_include :tempfile
     f[:tempfile].size.must_equal 76
+
+    req.env['rack.request.form_pairs'].must_equal [["reply", "yes"], ["fileupload", f]]
+  end
+
+  it "parse multipart delimiter-only boundary" do
+    input = <<EOF
+--AaB03x--\r
+EOF
+    mr = Rack::MockRequest.env_for(
+      "/",
+      "CONTENT_TYPE" => "multipart/form-data, boundary=AaB03x",
+      "CONTENT_LENGTH" => input.size,
+      :input => input
+    )
+
+    req = make_request mr
+    req.query_string.must_equal ""
+    req.GET.must_be :empty?
+    req.POST.must_be :empty?
+    req.params.must_equal({})
   end
 
   it "MultipartPartLimitError when request has too many multipart file parts if limit set" do
@@ -1589,6 +1577,10 @@ EOF
 
     parser.call("gzip ; q=0.9").must_equal [["gzip", 0.9]]
     parser.call("gzip ; deflate").must_equal [["gzip", 1.0]]
+
+    parser.call(", ").must_equal []
+    parser.call(", gzip").must_equal [["gzip", 1.0]]
+    parser.call("gzip, ").must_equal [["gzip", 1.0]]
   end
 
   it "parse Accept-Language correctly" do
@@ -1606,6 +1598,10 @@ EOF
 
     parser.call("fr ; q=0.9").must_equal [["fr", 0.9]]
     parser.call("fr").must_equal [["fr", 1.0]]
+
+    parser.call(", ").must_equal []
+    parser.call(", en").must_equal [["en", 1.0]]
+    parser.call("en, ").must_equal [["en", 1.0]]
   end
 
   def ip_app
